@@ -3,4 +3,4 @@
 #endif
 #include "system.h"
 #include "grep.h"
-char const default_matcher[] = "fgrep";
+char const *matcher = "fgrep";
