@@ -1,5 +1,5 @@
 /* grep.c - main driver file for grep.
-   Copyright (C) 1992, 1997, 1998, 1999 Free Software Foundation, Inc.
+   Copyright 1992, 1997-1999, 2000 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -60,7 +60,13 @@ static int mmap_option;
 
 /* Short options.  */
 static char const short_options[] =
-"0123456789A:B:C::EFGHUVX:abcd:e:f:hiLlnqrsuvwxyZz";
+"0123456789A:B:C::EFGHIUVX:abcd:e:f:hiLlnqrsuvwxyZz";
+
+/* Non-boolean long options that have no corresponding short equivalents.  */
+enum
+{
+  BINARY_FILES_OPTION = CHAR_MAX + 1
+};
 
 /* Long options equivalences. */
 static struct option long_options[] =
@@ -68,6 +74,7 @@ static struct option long_options[] =
   {"after-context", required_argument, NULL, 'A'},
   {"basic-regexp", no_argument, NULL, 'G'},
   {"before-context", required_argument, NULL, 'B'},
+  {"binary-files", required_argument, NULL, BINARY_FILES_OPTION},
   {"byte-offset", no_argument, NULL, 'b'},
   {"context", optional_argument, NULL, 'C'},
   {"count", no_argument, NULL, 'c'},
@@ -150,9 +157,7 @@ static char *(*execute) PARAMS ((char *, size_t, char **));
 /* Print a message and possibly an error string.  Remember
    that something awful happened. */
 static void
-error (mesg, errnum)
-     const char *mesg;
-     int errnum;
+error (const char *mesg, int errnum)
 {
   if (errnum)
     fprintf (stderr, "%s: %s: %s\n", prog, mesg, strerror (errnum));
@@ -163,9 +168,7 @@ error (mesg, errnum)
 
 /* Like error (), but die horribly after printing. */
 void
-fatal (mesg, errnum)
-     const char *mesg;
-     int errnum;
+fatal (const char *mesg, int errnum)
 {
   error (mesg, errnum);
   exit (2);
@@ -173,8 +176,7 @@ fatal (mesg, errnum)
 
 /* Interface to handle errors and fix library lossage. */
 char *
-xmalloc (size)
-     size_t size;
+xmalloc (size_t size)
 {
   char *result;
 
@@ -186,9 +188,7 @@ xmalloc (size)
 
 /* Interface to handle errors and fix some library lossage. */
 char *
-xrealloc (ptr, size)
-     char *ptr;
-     size_t size;
+xrealloc (char *ptr, size_t size)
 {
   char *result;
 
@@ -204,9 +204,7 @@ xrealloc (ptr, size)
 /* Convert STR to a positive integer, storing the result in *OUT.
    If STR is not a valid integer, return -1 (otherwise 0). */
 static int
-ck_atoi (str, out)
-     char const *str;
-     int *out;
+ck_atoi (char const *str, int *out)
 {
   char const *p;
   for (p = str; *p; p++)
@@ -250,9 +248,7 @@ static off_t initial_bufoffset;	/* Initial value of bufoffset. */
    possibly unaligned) buffer used to build the aligned buffer.  To
    free the buffer, free (*UP).  */
 static char *
-page_alloc (size, up)
-     size_t size;
-     char **up;
+page_alloc (size_t size, char **up)
 {
   size_t asize = size + pagesize - 1;
   if (size <= asize)
@@ -270,10 +266,7 @@ page_alloc (size, up)
 /* Reset the buffer for a new file, returning zero if we should skip it.
    Initialize on the first time through. */
 static int
-reset (fd, file, stats)
-     int fd;
-     char const *file;
-     struct stats *stats;
+reset (int fd, char const *file, struct stats *stats)
 {
   if (pagesize)
     bufsalloc = ALIGN_TO (bufalloc / PREFERRED_SAVE_FACTOR, pagesize);
@@ -342,9 +335,7 @@ reset (fd, file, stats)
    to the beginning of the buffer contents, and 'buflim'
    points just after the end.  Return zero if there's an error.  */
 static int
-fillbuf (save, stats)
-     size_t save;
-     struct stats *stats;
+fillbuf (size_t save, struct stats *stats)
 {
   size_t fillsize = 0;
   int cc = 1;
@@ -476,7 +467,12 @@ fillbuf (save, stats)
 }
 
 /* Flags controlling the style of output. */
-static int always_text;		/* Assume the input is always text. */
+static enum
+  {
+    BINARY_BINARY_FILES,
+    TEXT_BINARY_FILES,
+    WITHOUT_MATCH_BINARY_FILES
+  } binary_files;		/* How to handle binary files.  */
 static int filename_mask;	/* If zero, output nulls after filenames.  */
 static int out_quiet;		/* Suppress all normal output. */
 static int out_invert;		/* Print nonmatching stuff. */
@@ -505,8 +501,7 @@ static int done_on_match;		/* Stop scanning file on first match */
 #endif
 
 static void
-nlscan (lim)
-     char *lim;
+nlscan (char *lim)
 {
   char *beg;
   for (beg = lastnl;  (beg = memchr (beg, eolbyte, lim - beg));  beg++)
@@ -515,9 +510,7 @@ nlscan (lim)
 }
 
 static void
-print_offset_sep (pos, sep)
-     off_t pos;
-     int sep;
+print_offset_sep (off_t pos, int sep)
 {
   /* Do not rely on printf to print pos, since off_t may be longer than long,
      and long long is not portable.  */
@@ -534,10 +527,7 @@ print_offset_sep (pos, sep)
 }
 
 static void
-prline (beg, lim, sep)
-     char *beg;
-     char *lim;
-     int sep;
+prline (char *beg, char *lim, int sep)
 {
   if (out_file)
     printf ("%s%c", filename, sep & filename_mask);
@@ -563,8 +553,7 @@ prline (beg, lim, sep)
 
 /* Print pending lines of trailing context prior to LIM. */
 static void
-prpending (lim)
-     char *lim;
+prpending (char *lim)
 {
   char *nl;
 
@@ -584,10 +573,7 @@ prpending (lim)
 /* Print the lines between BEG and LIM.  Deal with context crap.
    If NLINESP is non-null, store a count of lines between BEG and LIM. */
 static void
-prtext (beg, lim, nlinesp)
-     char *beg;
-     char *lim;
-     int *nlinesp;
+prtext (char *beg, char *lim, int *nlinesp)
 {
   static int used;		/* avoid printing "--" before any output */
   char *bp, *p, *nl;
@@ -650,9 +636,7 @@ prtext (beg, lim, nlinesp)
    between matching lines if OUT_INVERT is true).  Return a count of
    lines printed. */
 static int
-grepbuf (beg, lim)
-     char *beg;
-     char *lim;
+grepbuf (char *beg, char *lim)
 {
   int nlines, n;
   register char *p, *b;
@@ -692,10 +676,7 @@ grepbuf (beg, lim)
    but if the file is a directory and we search it recursively, then
    return -2 if there was a match, and -1 otherwise.  */
 static int
-grep (fd, file, stats)
-     int fd;
-     char const *file;
-     struct stats *stats;
+grep (int fd, char const *file, struct stats *stats)
 {
   int nlines, i;
   int not_text;
@@ -729,11 +710,14 @@ grep (fd, file, stats)
     {
       if (! (is_EISDIR (errno, file) && suppress_errors))
 	error (filename, errno);
-      return nlines;
+      return 0;
     }
 
-  not_text = (! (always_text | out_quiet)
+  not_text = (((binary_files == BINARY_BINARY_FILES && !out_quiet)
+	       || binary_files == WITHOUT_MATCH_BINARY_FILES)
 	      && memchr (bufbeg, eol ? '\0' : '\200', buflim - bufbeg));
+  if (not_text && binary_files == WITHOUT_MATCH_BINARY_FILES)
+    return 0;
   done_on_match += not_text;
   out_quiet += not_text;
 
@@ -780,6 +764,7 @@ grep (fd, file, stats)
     }
   if (residue)
     {
+      *buflim++ = eol;
       nlines += grepbuf (bufbeg + save - residue, buflim);
       if (pending)
 	prpending (buflim);
@@ -794,9 +779,7 @@ grep (fd, file, stats)
 }
 
 static int
-grepfile (file, stats)
-     char const *file;
-     struct stats *stats;
+grepfile (char const *file, struct stats *stats)
 {
   int desc;
   int count;
@@ -890,9 +873,7 @@ grepfile (file, stats)
 }
 
 static int
-grepdir (dir, stats)
-     char const *dir;
-     struct stats *stats;
+grepdir (char const *dir, struct stats *stats)
 {
   int status = 1;
   struct stats *ancestor;
@@ -950,8 +931,7 @@ grepdir (dir, stats)
 }
 
 static void
-usage(status)
-int status;
+usage (int status)
 {
   if (status != 0)
     {
@@ -963,7 +943,7 @@ int status;
       printf (_("Usage: %s [OPTION]... PATTERN [FILE] ...\n"), prog);
       printf (_("\
 Search for PATTERN in each FILE or standard input.\n\
-Example: %s -i 'hello.*world' menu.h main.c\n\
+Example: %s -i 'hello world' menu.h main.c\n\
 \n\
 Regexp selection and interpretation:\n"), prog);
       printf (_("\
@@ -993,7 +973,10 @@ Output control:\n\
   -H, --with-filename       print the filename for each match\n\
   -h, --no-filename         suppress the prefixing filename on output\n\
   -q, --quiet, --silent     suppress all normal output\n\
-  -a, --text                do not suppress binary output\n\
+      --binary-files=TYPE   assume that binary files are TYPE\n\
+                            TYPE is 'binary', 'text', or 'without-match'.\n\
+  -a, --text                equivalent to --binary-files=text\n\
+  -I                        equivalent to --binary-files=without-match\n\
   -d, --directories=ACTION  how to handle directories\n\
                             ACTION is 'read', 'recurse', or 'skip'.\n\
   -r, --recursive           equivalent to --directories=recurse.\n\
@@ -1023,8 +1006,7 @@ and 2 if trouble.\n"));
 
 /* Set the matcher to M, reporting any conflicts.  */
 static void
-setmatcher (m)
-     char const *m;
+setmatcher (char const *m)
 {
   if (matcher && strcmp (matcher, m) != 0)
     fatal (_("conflicting matchers specified"), 0);
@@ -1034,8 +1016,7 @@ setmatcher (m)
 /* Go through the matchers vector and look for the specified matcher.
    If we find it, install it in compile and execute, and return 1.  */
 static int
-install_matcher (name)
-     char const *name;
+install_matcher (char const *name)
 {
   int i;
 #ifdef HAVE_SETRLIMIT
@@ -1083,10 +1064,7 @@ install_matcher (name)
    Do not set ARGV[N] to NULL.  If ARGV is NULL, do not store ARGV[0]
    etc.  Backslash can be used to escape whitespace (and backslashes).  */
 static int
-prepend_args (options, buf, argv)
-     char const *options;
-     char *buf;
-     char **argv;
+prepend_args (char const *options, char *buf, char **argv)
 {
   char const *o = options;
   char *b = buf;
@@ -1115,10 +1093,7 @@ prepend_args (options, buf, argv)
    vector of a main program with argument count *PARGC and argument
    vector *PARGV.  */
 static void
-prepend_default_options (options, pargc, pargv)
-     char const *options;
-     int *pargc;
-     char ***pargv;
+prepend_default_options (char const *options, int *pargc, char ***pargv)
 {
   if (options)
     {
@@ -1137,15 +1112,14 @@ prepend_default_options (options, pargc, pargv)
 }
 
 int
-main (argc, argv)
-     int argc;
-     char *argv[];
+main (int argc, char **argv)
 {
   char *keys;
   size_t keycc, oldcc, keyalloc;
   int with_filenames;
   int opt, cc, status;
-  unsigned digit_args_val, default_context;
+  int default_context;
+  unsigned digit_args_val;
   FILE *fp;
   extern char *optarg;
   extern int optind;
@@ -1259,6 +1233,9 @@ main (argc, argv)
       case 'H':
 	with_filenames = 1;
 	break;
+      case 'I':
+	binary_files = WITHOUT_MATCH_BINARY_FILES;
+	break;
       case 'U':
 #if O_BINARY
 	dos_use_file_type = DOS_BINARY;
@@ -1276,7 +1253,7 @@ main (argc, argv)
 	setmatcher (optarg);
 	break;
       case 'a':
-	always_text = 1;
+	binary_files = TEXT_BINARY_FILES;
 	break;
       case 'b':
 	out_byte = 1;
@@ -1370,6 +1347,16 @@ main (argc, argv)
       case 'z':
 	eolbyte = '\0';
 	break;
+      case BINARY_FILES_OPTION:
+	if (strcmp (optarg, "binary") == 0)
+	  binary_files = BINARY_BINARY_FILES;
+	else if (strcmp (optarg, "text") == 0)
+	  binary_files = TEXT_BINARY_FILES;
+	else if (strcmp (optarg, "without-match") == 0)
+	  binary_files = WITHOUT_MATCH_BINARY_FILES;
+	else
+	  fatal (_("unknown binary-files type"), 0);
+	break;
       case 0:
 	/* long options */
 	break;
@@ -1391,7 +1378,7 @@ main (argc, argv)
       printf (_("%s (GNU grep) %s\n"), matcher, VERSION);
       printf ("\n");
       printf (_("\
-Copyright (C) 1988, 1992-1998, 1999 Free Software Foundation, Inc.\n"));
+Copyright 1988, 1992-1999, 2000 Free Software Foundation, Inc.\n"));
       printf (_("\
 This is free software; see the source for copying conditions. There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"));
